@@ -4,12 +4,17 @@
 
 from argparse import ArgumentError
 from datetime import datetime
-from typing import Any, Iterable, List, Optional, Tuple, Union
+from typing import Any, Dict, Iterable, List, Optional, Tuple, Union
 
 import grpc
 from google.protobuf import timestamp_pb2 as pbts
 
-from cloudvision.Connector import codec as codec
+from cloudvision import __version__ as version
+from cloudvision.Connector import codec
+from cloudvision.Connector.auth.cert import gen_csr_der
+from cloudvision.Connector.core.utils import get_dict
+from cloudvision.Connector.gen import ca_pb2 as ca
+from cloudvision.Connector.gen import ca_pb2_grpc as ca_client
 from cloudvision.Connector.gen import notification_pb2 as ntf
 from cloudvision.Connector.gen import router_pb2 as rtr
 from cloudvision.Connector.gen import router_pb2_grpc as rtr_client
@@ -105,6 +110,10 @@ class GRPCClient(object):
     """
 
     AUTH_KEY_PATH = "access_token"
+    DEFAULT_CHANNEL_OPTIONS = {
+        "grpc.primary_user_agent": f"cloudvision.Connector/{version}",
+        "grpc.keepalive_time_ms": 60000,  # 60 seconds
+    }
 
     def __init__(
         self,
@@ -117,13 +126,20 @@ class GRPCClient(object):
         tokenValue: Optional[str] = None,
         certsValue: Optional[str] = None,
         keyValue: Optional[str] = None,
-        caValue: Optional[str] = None
+        caValue: Optional[str] = None,
+        channel_options: Dict[str, Any] = {},
     ) -> None:
         # used to store the auth token for per request auth
         self.metadata = None
+        # create a channel option by merging the default channel options and the user provided one
+        # NOTE: default options will be overriden if provided by the user
+        channel_options = get_dict(channel_options)
+        self.channel_options = [
+            (k, v) for k, v in dict(GRPCClient.DEFAULT_CHANNEL_OPTIONS, **channel_options).items()
+        ]
 
         if (certs is None or key is None) and (token is None and tokenValue is None):
-            self.channel = grpc.insecure_channel(grpcAddr)
+            self.channel = grpc.insecure_channel(grpcAddr, options=self.channel_options)
         else:
             tokCreds = None
             if token or tokenValue:
@@ -181,10 +197,11 @@ class GRPCClient(object):
             if tokCreds:
                 creds = grpc.composite_channel_credentials(creds, tokCreds)
 
-            self.channel = grpc.secure_channel(grpcAddr, creds)
+            self.channel = grpc.secure_channel(grpcAddr, creds, options=self.channel_options)
         self.__client = rtr_client.RouterV1Stub(self.channel)
         self.__auth_client = rtr_client.AuthStub(self.channel)
         self.__search_client = rtr_client.SearchStub(self.channel)
+        self.__ca_client = ca_client.CertificateAuthorityStub(self.channel)
 
         self.encoder = codec.Encoder()
         self.decoder = codec.Decoder()
@@ -362,3 +379,19 @@ class GRPCClient(object):
         )
         res = self.__search_client.Search(req)
         return (self.decode_batch(nb) for nb in res)
+
+    def reenroll(self, cert_path: str, key_path: str) -> bytes:
+        """Reenroll the existing certificate.
+
+        Caller can pass their current key and certificate and receive a renewed
+        certificate in return.
+        The default validity of the renewed certificate is 90 days.
+
+        :param cert_path: Certificate file path
+        :param key_path: Private Key file path
+        :return: Renewed Certificate in ASN.1 DER
+        """
+
+        csr_der: bytes = gen_csr_der(cert_path, key_path)
+        crt = self.__ca_client.Reenroll(ca.CSR(csr=csr_der))
+        return crt.crt
